<?php

namespace Teknisa\FIN\Modules\ContaReceber\Controllers;

use Financeiro\Entities\GeralRec;
use Financeiro\Logic\ContasReceber\Boleto\AssociarBoleto\ValidacaoAssociar;
use Teknisa\Libs\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Teknisa\FIN\Modules\ContaReceber\Services\TitulosAReceberService as TitulosAReceberService;

/**
 * Class TitulosAReceberController
 * @package Teknisa\FIN\Modules\ContaReceber\Controllers
 */
class TitulosAReceberController extends TeknisaCrud{

    protected $dataSourceName = 'boletreceber_tipobaixarec_contcorr';

    protected $titulosAReceberService;

    private function existemDadosDoBoleto($row){
        return (isset($row['VRMULTABREC']) &&
            isset($row['VRMORDIABREC']) &&
            isset($row['CDREDUCONT']) &&
            isset($row['CDNOSNUMREC']) &&
            isset($row['CDBARRAREC']) &&
            isset($row['DSLINHABARREC']) &&
            isset($row['CDTPBAIXAREC'])
            );
    }

    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param TitulosAReceberService $titulosAReceberService
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, TitulosAReceberService $titulosAReceberService)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->titulosAReceberService = $titulosAReceberService;
    }

    /**
     * Controi Linha Titutlos a Receber
     * @param $row
     * @return array
     * @throws \Exception
     */
    private function buildTitulosAReceberRow($row){
        $completeRow = array();

        // Parâmetros Iniciais
        $completeRow['NRORG'] = $row['NRORG'];
        $completeRow['CDFILIAL'] = $row['CDFILIAL'];
        $completeRow['CDFILIEMIT'] = $row['CDFILIEMIT'];
        $completeRow['CDEMPRESAEMIT'] = $row['CDEMPRESAEMIT'];
        $completeRow['CDINSCESTAEMIT'] = $row['CDINSCESTAEMIT'];
        $completeRow['CDINSCMUNIEMIT'] = $row['CDINSCMUNIEMIT'];
        $completeRow['CDCLIENTE'] = $row['CDCLIENTE'];
        $completeRow['NRINSJURCLIE'] = $row['NRINSJURCLIE'];
        $completeRow['IDTPIJURCLIE'] = $row['IDTPIJURCLIE'];
        $completeRow['NRLICENCAUSO'] = isset($row['NRLICENCAUSO'])?$row['NRLICENCAUSO']:null;
        $completeRow['NRNOTAFISC'] = $row['NRNOTAFISC'];
        $completeRow['CDSERIE'] = $row['CDSERIE'];
        $completeRow['IDSTATUS'] = $row['IDSTATUS'];
        $completeRow['CDTIPOOPER'] = $row['CDTIPOOPER'];
        $completeRow['CDVENDEDOR'] = isset($row['CDVENDEDOR'])?$row['CDVENDEDOR']:null;
        $completeRow['VRNOTAFISC'] = $row['VRNOTAFISC'];
        $completeRow['CDTPCTRECEB'] = $row['CDTPCTRECEB'];
        $completeRow['DSCONTAREC'] = $row['DSCONTAREC'];
        $completeRow['CDCLASSFINA'] = $row['CDCLASSFINA'];
        if(isset($row['CDREDUCONT'])) $completeRow['CDREDUCONT'] = $row['CDREDUCONT'];
        $completeRow['DATAATUAL']  = DateUtil::getDataAtual();
        $completeRow['DTEMISSAO']  = $this->titulosAReceberService->formataData($row['DTEMISSAO'], true);
        $completeRow['DTENTRSAID'] = $this->titulosAReceberService->formataData($row['DTENTRSAID'], true);
        $completeRow['DTATUAVENREC']  = $this->titulosAReceberService->formataData($row['DTATUAVENREC'], true);

        // Parâmetros Nota Fiscal
        $completeRow['NRINSJURNF'] = $completeRow['NRINSJURCLIE'];
        $completeRow['IDTPIJURNF'] = $completeRow['IDTPIJURCLIE'];
        $completeRow['CDEMPRESA'] = $completeRow['CDEMPRESAEMIT'];
        $completeRow['CDINSCESTA'] = $completeRow['CDINSCESTAEMIT'];
        $completeRow['CDINSCMUNI'] = $completeRow['CDINSCMUNIEMIT'];
        $completeRow['VRPRODUTOS'] = $completeRow['VRNOTAFISC'];
        $completeRow['IDBLOQUEADO'] = 'S';
        $completeRow['IDDOCFISCAL'] = 'N';
        $completeRow['IDDOCFINANC'] = 'S';
        $completeRow['IDENTRSAIDOP'] = 'S';
        $completeRow['IDDOCCONTAB'] = 'S';

        // Parâmetros Boleto
        $completeRow['NRLANBOLREC'] = $this->titulosAReceberService->geraNovoCodigo($completeRow['NRORG'], 'BOLETRECEBER'.$completeRow['CDFILIEMIT']);
        $completeRow['IDDOCFISCENT']    = 'N';
        $completeRow['IDDOCFISCSAI']    = 'S';
        $completeRow['IDDOCESTOQUE']    = 'N';
        $completeRow['IDSISTEMA']       = 'REC';
        $completeRow['DTLANCAMENNF']    = $completeRow['DATAATUAL'];
        $completeRow['VRBOLREC']        = $completeRow['VRNOTAFISC'];
        $completeRow['VRFINALBREC']     = $completeRow['VRNOTAFISC'];
        $completeRow['DTVENBOLREC']     = $completeRow['DTATUAVENREC'];
        $completeRow['DSBOLETREC']      = $completeRow['DSCONTAREC'];

        // Parâmetros Movimentação Fluxo
        $completeRow['DTMOVFLUXO']      = $completeRow['DTATUAVENREC'];
        $completeRow['IDMOVFLUXO']      = ($completeRow['IDSTATUS'] == 'V')?'P':'E';

        // Parâmetros Conta
        $completeRow['NRSEQVENREC'] = (isset($row['NRSEQVENREC']))? $row['NRSEQVENREC']: '01';
        $completeRow['NRPARCELAREC']    = '00';
        $completeRow['DTORIGVENREC']    = $completeRow['DTATUAVENREC'];
        $completeRow['VRATUAREC']       = $completeRow['VRNOTAFISC'];
        $completeRow['VRORIGREC']       = $completeRow['VRNOTAFISC'];
        $completeRow['CDFILIBOLREC']    = $completeRow['CDFILIEMIT'];
        $completeRow['IDSTATREC']       = 'A';
        $completeRow['QTDIASVENREC']    = 0;
        $completeRow['CDFIMOVFLREC']    = $completeRow['CDFILIEMIT'];

        if(self::existemDadosDoBoleto($row)){
            $completeRow['VRMULTABREC'] = $row['VRMULTABREC'];
            $completeRow['VRMORDIABREC'] = $row['VRMORDIABREC'];
            $completeRow['CDREDUCONT'] = $row['CDREDUCONT'];
            $completeRow['CDNOSNUMREC'] = $row['CDNOSNUMREC'];
            $completeRow['CDBARRAREC'] = $row['CDBARRAREC'];
            $completeRow['DSLINHABARREC'] = $row['DSLINHABARREC'];
            $completeRow['CDTPBAIXAREC'] = $row['CDTPBAIXAREC'];
        }
        return $completeRow;
    }

    /**
     * Salva Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function salvaTituloAReceber(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $rows = $row->getRow();

            $permiteParcelamento = isset($rows["PARCELAMENTO"]);

            // Configura a row para enviar para o service
            $row = self::buildTitulosAReceberRow($rows);

            // Salva Titulo a Receber
            $this->titulosAReceberService->geraTituloAReceber($row, $response, $permiteParcelamento);

            // Registra Dados
            $response->addDataSet(new DataSet('TitulosAReceberService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Titulos a Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function geraTitulosAReceber(Data $dataset, Response $response){
        try {
            // Dados do Registro
            $rows = $dataset->getDataSet()->getRows();
            $permiteParcelamento = isset($rows[0]["PARCELAMENTO"]);

            // Configura as rows para enviar para o service
            foreach($rows as &$row){
                $row = self::buildTitulosAReceberRow($row);
            }

            // Salva Titulo a Receber
            $this->titulosAReceberService->geraTitulosAReceber($rows, $response, $permiteParcelamento);

            // Registra Dados
            $response->addDataSet(new DataSet('TitulosAReceberService', ['success' => 'S']));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Edita Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function editaTituloAReceber(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $rows = $row->getRow();

            // Parâmetros Iniciais
            $nrorg         = $rows['NRORG'];
            $cdfilial      = $rows['CDFILIAL'];
            $nrlanbolrec   = $rows['NRLANBOLREC'];
            $cdreducont    = $rows['CDREDUCONT'];
            $cdtpbaixarec  = $rows['CDTPBAIXAREC'];
            $cdcartcobbol  = $rows['CDCARTCOBBOL'];
            $idcartcobrec  = $rows['IDCARTCOBREC'];
            $idimprbolbol  = $rows['IDIMPRBOLBOL'];
            $idformcadbrec = $rows['IDFORMCADBREC'];
            $cdins1cobbol  = $rows['CDINS1COBBOL'];
            $cdins2cobbol  = $rows['CDINS2COBBOL'];
            $qtdiaprotbol  = $rows['QTDIAPROTBOL'];
            $cddiaprotbol  = $rows['CDDIAPROTBOL'];
            $vrpemulbrec   = $rows['VRPEMULBREC'];
            $vrpemorbrec   = $rows['VRPEMORBREC'];
            $vrmultabrec   = $rows['VRMULTABREC'];
            $vrmordiabrec  = $rows['VRMORDIABREC'];
            $dsboletrec    = $rows['DSBOLETREC'];

            // Edita Titulo a Receber
            $this->titulosAReceberService->editaTituloAReceber
            (
                $nrorg, $cdfilial, $nrlanbolrec, $cdreducont, $cdtpbaixarec, $cdcartcobbol, $idcartcobrec,
                $idimprbolbol, $idformcadbrec, $cdins1cobbol, $cdins2cobbol, $qtdiaprotbol, $cddiaprotbol,
                $vrpemulbrec, $vrpemorbrec, $vrmultabrec, $vrmordiabrec, $dsboletrec
            );

            // Registra Dados
            $response->addDataSet(new DataSet('TitulosAReceberService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Atualiza Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function atualizaTituloAReceber(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $rows = $row->getRow();

            // Parâmetros Iniciais
            $nrorg           = $rows['NRORG'];
            $cdfilial        = $rows['CDFILIAL'];
            $nrnotafisc      = $rows['NRNOTAFISC'];
            $cdserie         = $rows['CDSERIE'];
            $idstatus        = $rows['IDSTATUS'];
            $cdtpctreceb     = $rows['CDTPCTRECEB'];
            $dscontarec      = $rows['DSCONTAREC'];
            $cdclassfina     = $rows['CDCLASSFINA'];
            $nrlanctonf      = $rows['NRLANCTONF'];

            $dtemissao       = $this->titulosAReceberService->formataData($rows['DTEMISSAO']);
            $dtentrsaid      = $this->titulosAReceberService->formataData($rows['DTENTRSAID']);

            // Parâmetros Movimentação Fluxo
            $idmovfluxo      = ($idstatus == 'V')?'P':'E';

            // Atualiza Titulo a Receber
            $this->titulosAReceberService->atualizaTituloAReceber(
                $nrorg, $cdfilial, $nrnotafisc, $dtemissao, $dtentrsaid, $nrlanctonf,
                $cdserie, $idstatus, $idmovfluxo, $cdclassfina,
                $cdtpctreceb, $dscontarec, $response
            );

            // Registra Dados
            $response->addDataSet(new DataSet('atualizaTituloAReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Valida Associar Titulo A Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function validaAssociarTituloAReceber(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $this->titulosAReceberService->validaAssociacao($boletos);

            // Registra Dados
            $response->addDataSet(new DataSet('TitulosAReceberService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Associar Titulo a Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function associarTituloAReceber(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $nrorg        = $boletos[0]['nrorg'];
            $cdfilial     = $boletos[0]['cdfilial'];
            $dtvenbolrec  = $boletos[0]['dtvenbolrec'];
            $cdreducont   = $boletos[0]['cdreducont'];
            $cdtpbaixarec = $boletos[0]['cdtpbaixarec'];

            $boletos = $boletos[1];

            $this->titulosAReceberService->validaDataVencimento($dtvenbolrec, $boletos);

            $this->titulosAReceberService->geraTituloAssociado(
                $nrorg, $cdfilial, $boletos, $dtvenbolrec, $cdreducont, $cdtpbaixarec
            );

            // Registra Dados
            $response->addDataSet(new DataSet('TitulosAReceberService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Cancela Associação Titulo a Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function cancelaAssociarTituloARec(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();
            $nrorg       = $boletos[0]['nrorg'];

            foreach ($boletos[1] as $boleto) {
                $cdfilial    = $boleto["CDFILIAL"];
                $nrlanbolrec = $boleto["NRLANBOLREC"];
                $this->titulosAReceberService->cancelaAssociarTituloARec($nrorg, $cdfilial, $nrlanbolrec);
            }

            // Registra Dados
            $response->addDataSet(new DataSet('cancelaAssociarTituloARec', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Valida Associar Titulo a Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function validaProrrogarTituloAReceber(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $cnab = $this->titulosAReceberService->verificaSitCNAB($boletos);

            // Registra Dados
            $response->addDataSet(new DataSet('validaProrrogar', ['cnab' => $cnab, 'success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Prorrogar Titulo a Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function prorrogarTituloAReceber(Data $dataset, Response $response)
    {
        try {
            $rows  = $dataset->getDataSet()->getRows();

            $nrorg       = $this->environment->getNrOrgTrab();
            $idtppror    = $rows[0]['IDTPPROR'];
            $dtvenbolrec = $rows[0]['DTVENBOLREC'];
            $nrdiasvenc  = $rows[0]['NRDIASVENC'];
            $cdmotipror  = $rows[0]['CDMOTIPROR'];
            $titulos     = $rows[1];
            
            $this->titulosAReceberService->prorrogaTituloAReceber($nrorg, $idtppror, $dtvenbolrec, $nrdiasvenc, $cdmotipror, $titulos);

            $response->addDataSet(new DataSet('prorrogarTituloAReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Emitir Titulo a Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function emitirTituloAReceber(Data $dataset, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $relatorios   = array();
            $nrorg       = $boletos[0]['nrorg'];

            foreach ($boletos[1] as $boleto) {

                $cdfilial    = $boleto["CDFILIAL"];
                $nrlanbolrec = $boleto["NRLANBOLREC"];

                if(!$boleto['CDREDUCONT'])
                    throw new \Exception('Título sem conta corrente cadastrada.');

                $relatorionome = $this->titulosAReceberService->emiteBoleto($nrorg, $cdfilial, $nrlanbolrec);
                $relatorios[] = ['NRORG' => $nrorg, 'CDFILIAL' => $cdfilial, 'NRLANBOLREC' => $nrlanbolrec, 'RELATORIO' => $relatorionome];
            }
            // Registra Dados
            $response->addDataSet(new DataSet('EmitirTituloAReceber', ['relatorios' => $relatorios]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Gera Ajuste Títulos
     * @param Data $dataset
     * @param Response $response
     */
    public function salvarAjusteBoleto(Data $dataset, Response $response)
    {
        try {
            $dataset  = $dataset->getDataSet()->getRows();
            $boleto = $dataset[0];
            $ajuste = $dataset[1];
            $ajuste['VRAJBOLREC'] = floatval($ajuste['VRAJBOLREC']);
            $nrorg = $this->environment->getNrOrgTrab();
            $cdfilial = $this->environment->getCdfilial();
            $cdoperador = $this->environment->getCdoperador();

            $this->titulosAReceberService->salvarAjusteBoleto($nrorg, $cdfilial, $cdoperador, $boleto, $ajuste);
            $response->addDataSet(new DataSet('AjusteBoletoService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Finaliza Parcelamento Títulos
     * @param Data $dataset
     * @param Response $response
     */
    public function finalizaParcelamentoTitulos(Data $dataset, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            // Dados do Registro
            $dataset  = $dataset->getDataSet()->getRows();
            $boleto = $dataset[0];
            $parcelamento = $this->preparaDataParcelamento($dataset[1]);
            $nrorg = $this->environment->getNrOrgTrab();
            $cdfilial = $this->environment->getCdfilial();
            $cdoperador = $this->environment->getCdoperador();

            $this->titulosAReceberService->finalizaParcelamentoTitulos($nrorg, $cdfilial, $cdoperador, $boleto, $parcelamento);
            $response->addDataSet(new DataSet('ParcelamentoBoletoService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function preparaDataParcelamento($parcelamento){
        $parcelas = array();
        foreach($parcelamento as $data){
            $parcelas[] = array(
                'NRORG'            => $data['NRORG']       ,
                'CDFILIAL'         => $data['CDFILIAL']    ,
                'NRLANBOLREC'      => $data['NRLANBOLREC'] ,
                'NRORIGBOLREC'     => $data['NRORIGBOLREC'],
                'NRPARCBOLREC'     => $data['NRPARCBOLREC'],
                'NRPARCBOLRECVISU' => $data['NRPARCBOLRECVISU'],
                'DTVENBOLREC'      => $data['DTVENBOLREC'] ,
                'VRFINALBREC'      => floatval($data['VRFINALBREC']) ,
                'CDMOTIPARC'       => $data['CDMOTIPARC'],
                'IDSTATBREC'       => $data['IDSTATBREC']
            );
        }
        return $parcelas;
    }

    /**
     * Cancela Emissão Titulo a Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function cancelaEmissaoTituloARec(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $nrorg          = $boletos[0]['nrorg'];
            $mantemnossonum = $boletos[0]['mantemnossonum'];

            foreach ($boletos[1] as $boleto) {

                $cdfilial    = $boleto["CDFILIAL"];
                $nrlanbolrec = $boleto["NRLANBOLREC"];

                $this->titulosAReceberService->cancelaEmissaoTituloARec($nrorg, $cdfilial, $nrlanbolrec, $mantemnossonum);
            }
            // Registra Dados
            $response->addDataSet(new DataSet('CancelaEmissaoTituloAReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Verifica CNAB Emissão Titulo A Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function verificaCnabTituloAReceber(Data $dataset, Response $response)
    {
        try {
            $boletos  = $dataset->getDataSet()->getRows();

            $exibiconf = $this->titulosAReceberService->verificaCnabTituloAReceber($boletos);


            $response->addDataSet(new DataSet('verificaCnabTituloAReceber', ['exibiconf' => $exibiconf]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Associar Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function geraNovoCodigo(Row $row, Response $response)
    {
        try {
            $row  = $row->getRow();

            $nrorg      = $row['NRORG'];
            $cdcontador = $row['CDCONTADOR'];
            $intervalo  = isset($row['INTERVALO']) ? $row['INTERVALO'] : 1;
            $tamanho    = isset($row['TAMANHO'])   ? $row['TAMANHO']   : 10;

            $novocodigo = $this->titulosAReceberService->geraNovoCodigo($nrorg, $cdcontador, $intervalo, $tamanho);
            $response->setDataSets(array(new DataSet('geraNovoCodigo', ['novocodigo' => $novocodigo])));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Deleta Vencimento Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function deletaVencTituloReceber(Row $row, Response $response)
    {
        try {
            $row  = $row->getRow();

            $nrorg       = $row['NRORG'];
            $cdfilial    = $row['CDFILIAL'];
            $nrlanbolrec = $row['NRLANBOLREC'];

            $this->titulosAReceberService->deletaVencTituloReceber($nrorg, $cdfilial, $nrlanbolrec, $response);

            $response->addDataSet(new DataSet('deletaVencTituloReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Deleta Documento Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function deletaDocTituloReceber(Row $row, Response $response)
    {
        try {
            $row  = $row->getRow();

            $nrorg       = $row['NRORG'];
            $cdfilial    = $row['CDFILIAL'];
            $nrlanbolrec = $row['NRLANBOLREC'];

            $this->titulosAReceberService->deletaDocTituloReceber($nrorg, $cdfilial, $nrlanbolrec);

            $response->addDataSet(new DataSet('deletaDocTituloReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Remessa Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function remessaTituloReceber(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $remessaboleto   = $row['remessaboleto'];
            $boletosmarcados = $row['boletosmarcados'];
            
            $dsarqcnabrec = $this->titulosAReceberService->remessaTituloReceber($nrorg, $cdfilial, $remessaboleto, $boletosmarcados);

            $response->addDataSet(new DataSet('remessaTituloReceber', ['dsarqcnabrec' => $dsarqcnabrec]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Baixa Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function baixaTituloReceber(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $baixaboleto     = $row['baixaboleto'];
            $boletosmarcados = $row['boletosmarcados'];
            $ajustesboletos  = $row['ajustesboletos'];
            $contabiliza     = isset($row['contabiliza']) ? $row['contabiliza'] : true;
            
            if($baixaboleto['IDUNICABAIXA'] == 'N')
                foreach ($boletosmarcados as $boleto)
                    $this->titulosAReceberService->baixaTituloReceber($nrorg, $cdfilial, $baixaboleto, [$boleto], $response, $ajustesboletos, $contabiliza);
            else $this->titulosAReceberService->baixaTituloReceber($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response, $ajustesboletos, $contabiliza);

            $response->addDataSet(new DataSet('baixaTituloReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclusão Baixa Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function excluiBaixaTituloReceber(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $boletosmarcados = $row['boletosmarcados'];

            $this->titulosAReceberService->excluiBaixaTituloReceber($nrorg, $cdfilial, $boletosmarcados);

            $response->addDataSet(new DataSet('excluiBaixaTituloReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Transforma Planejado Titulo
     * @param Row $row
     * @param Response $response
     */
    public function transformaPlanejadoTitulo(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $baixaboleto     = $row['baixaboleto'];
            $boletosmarcados = $row['boletosmarcados'];

            $this->titulosAReceberService->transformaPlanejadoTitulo($nrorg, $cdfilial, $boletosmarcados, $baixaboleto, $response);

            $response->addDataSet(new DataSet('transformaPlanejadoTitulo', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclusão Boleto Baixa Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function excluiBoletoBaixaTituloReceber(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);

            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $boletosmarcados = $row['boletosmarcados'];

            $this->titulosAReceberService->excluiBoletoBaixaTituloReceber($nrorg, $cdfilial, $boletosmarcados, $response);

            $response->addDataSet(new DataSet('excluiBaixaTituloReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Transforma Boleto Planejado Titulo
     * @param Row $row
     * @param Response $response
     */
    public function transformaBoletoPlanejadoTitulo(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            $nrorg           = $row['nrorg'];
            $cdfilial        = $row['cdfilial'];
            $baixaboleto     = $row['baixaboleto'];
            $boletosmarcados = $row['boletosmarcados'];

            $this->titulosAReceberService->transformaBoletoPlanejadoTitulo($nrorg, $cdfilial, $boletosmarcados, $baixaboleto, $response);

            $response->addDataSet(new DataSet('transformaBoletoPlanejadoTitulo', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Título a Receber da Remessa
     * @param Row $row
     * @param Response $response
     */
    public function excluiTituloRecRemessa(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);

            $row  = $row->getRow();

            $nrorg      = $row[0]['nrorg'];
            $cdfilial   = $row[0]['cdfilial'];
            $cdoperador = $row[0]['cdoperador'];
            $boletos    = $row[1];

            $this->titulosAReceberService->excluiTituloRecRemessa($nrorg, $cdfilial, $cdoperador, $boletos);

            $response->addDataSet(new DataSet('excluiTituloRecRemessa', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Remessa Receber
     * @param Row $row
     * @param Response $response
     */
    public function excluiRemessaRec(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            set_time_limit(0);

            $row  = $row->getRow();

            $nrorg      = $row[0]['nrorg'];
            $cdfilial   = $row[0]['cdfilial'];
            $cdoperador = $row[0]['cdoperador'];
            $boletos    = $row[1];

            $this->titulosAReceberService->excluiRemessaRec($nrorg, $cdfilial, $cdoperador, $boletos);

            $response->addDataSet(new DataSet('excluiRemessaRec', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Exclui Título a Receber da Remessa
     * @param Row $row
     * @param Response $response
     */
    public function downloadRemessaReceber(Row $row, Response $response)
    {
        try {
            $row  = $row->getRow();

            $local_arquivo = $row['local_arquivo'];

            $this->titulosAReceberService->downloadRemessaReceber($local_arquivo);

            $response->addDataSet(new DataSet('downloadRemessaReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Adiantamento Cliente
     * @param Data $dataset
     * @param Response $response
     */
    public function geraAdiantamentoCliente(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $tituloGrid         = $rows[0];
            $adiantamentoTabbed = $rows[1];
            $adiantamentoGrid   = $rows[2];

            // Gera Aditamento Fornencedor
            $this->titulosAReceberService->geraAdiantamentoCliente(
                $tituloGrid, $adiantamentoTabbed, $adiantamentoGrid
            );

            // Registra Dados
            $response->addDataSet(new DataSet('geraAdiantamentoCliente', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Devolução Cliente
     * @param Data $dataset
     * @param Response $response
     */
    public function geraDevolucaoCliente(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $tituloGrid      = $rows[0];
            $devolucaoTabbed = $rows[1];
            $devolucaoGrid   = $rows[2];

            // Gera Devolução Fornencedor
            $this->titulosAReceberService->geraDevolucaoCliente(
                $tituloGrid, $devolucaoTabbed, $devolucaoGrid
            );

            // Registra Dados
            $response->addDataSet(new DataSet('geraDevolucaoCliente', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Envia Título Por Email
     * @param Data $dataset
     * @param Response $response
     */
    public function enviaTituloPorEmail(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $boletos = $rows[0];
            $envio   = $rows[1];

            $remetente = $envio['DSEMAILFILI'];
            $assunto   = $envio['DSASSUEMAIL'];
            $mensagem  = $envio['DSMENSEMAIL'];
            
            // Envia Título Por Email
            $this->titulosAReceberService->enviaTituloPorEmail(
                $remetente, $assunto, $mensagem, $boletos, $dataset, $response
            );

            // Registra Dados
            $response->addDataSet(new DataSet('enviaTituloPorEmail', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Cadastramento Automatico
     * @param Row $row
     * @param Response $response
     */
    public function geraCadastramentoAutomatico(Row $row, Response $response){
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $row  = $row->getRow();

            // Gera Data Source Cadastramento Automatico
            $rows = self::geraDataSourceCadastramentoAutomatico($row);

            // Configura as rows para enviar para o service
            foreach($rows as &$row){
                $row = self::buildTitulosAReceberRow($row);
            }

            // Gera Títulos a Receber
            $this->titulosAReceberService->geraTitulosAReceber($rows, $response, false);

            // Registra Dados
            $response->addDataSet(new DataSet('geraCadastramentoAutomatico', ['success' => 'S']));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Data Source Cadastramento Automático
     * @param $row
     * @return array
     * @throws \Exception
     */
    public function geraDataSourceCadastramentoAutomatico($row){
        $rows = [];
        try {
            $row['NRLICENCAUSO'] = null;
            $row['CDVENDEDOR']   = null;
            $row['DSDEFACTREC']  = isset($row['DSDEFACTREC'])?$row['DSDEFACTREC']:$row['DSTPCTRECEB'];
            $row['DSCONTAREC']   = isset($row['DSCONTAREC'])?$row['DSCONTAREC']:$row['DSDEFACTREC'];

            for($i = 0; $i < $row['NRTITULOS']; $i++) {
                $row['NRNOTAFISC']   = $this->titulosAReceberService->geraNovoCodigo($row['NRORG'], 'NRNOTAFISCREC', 1, $row['NRTAMNUMNFUT']);
                $row['DTEMISSAO']    = $row['DTPRIMVENC'];
                $row['DTENTRSAID']   = $row['DTPRIMVENC'];
                $row['DTATUAVENREC'] = $row['DTPRIMVENC'];

                $rows[] = clone $row;
                
                if (strlen($row['DTPRIMVENC']) == 19)
                    $row['DTPRIMVENC'] = DateUtil::truncateString($row['DTPRIMVENC']);

                $row['DTPRIMVENC'] = (($row['IDPROXVENC'] == 'F')
                                        ? DateUtil::truncateString(DateUtil::adicionaIntervalo(DateUtil::truncateData(DateUtil::getDataDeString($row['DTPRIMVENC'])), 1, 'M'))
                                        : DateUtil::truncateString(DateUtil::adicionaIntervalo(DateUtil::truncateData(DateUtil::getDataDeString($row['DTPRIMVENC'])), $row['NRDIASVENC'], 'D')));
            }
        } catch (\Exception $e) {
            throw $e;
        }
        return $rows;
    }

    /**
     * Confirma Renegociacao Financeira
     * @param Data $dataset
     * @param Response $response
     */
    public function confirmaRenegociacaoFinanceira(Data $dataset, Response $response) {
        try {
            // Timeout Ilimitado
            set_time_limit(0);
            
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $renegociacao = $rows[0];
            $titulosReneg = $rows[1];
            $titulosMarca = $rows[2];

            // Gera Titulos Renegociação Financeira
            $rows = self::geraTitulosRenegociacaoFinanceira($renegociacao, $titulosReneg);

            // Gera Títulos a Receber
            $renegociacao['NRLANCTONF'] = $this->titulosAReceberService->geraNovoCodigo($renegociacao['NRORG'], 'LANCNF/FILIAL'.$renegociacao['CDFILIAL']);

            // Configura Titulos para Salvamento
            foreach($rows as &$row){
                $row = self::buildTitulosAReceberRow($row);
                $row['NRLANCTONF']  = $renegociacao['NRLANCTONF'];
            }

            // Gera Renegociação Titulos
            $this->titulosAReceberService->geraRenegociacaoTitulos($rows, $renegociacao, $titulosReneg, $titulosMarca, $response);

            // Registra Dados
            $response->addDataSet(new DataSet('confirmaRenegociacaoFinanceira', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Salva Titulo Descontado
     * @param Data $dataset
     * @param Response $response
     */
    public function salvaTituloDescontado(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            $dadosDescontados   = $rows[0];
            $titulosDescontados = $rows[1];

            // Salva Titulo Descontado
            $this->titulosAReceberService->salvaTituloDescontado(
                $dadosDescontados, $titulosDescontados
            );

            // Registra Dados
            $response->addDataSet(new DataSet('salvaTituloDescontado', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Gera Data Source Cadastramento Automático
     * @param $row
     * @return array
     * @throws \Exception
     */
    public function geraTitulosRenegociacaoFinanceira($renegociacao, $titulosReneg){
        $rows = [];
        try {

            $row['NRORG']          = $renegociacao['NRORG'];
            $row['CDFILIAL']       = $renegociacao['CDFILIAL'];
            $row['CDFILIEMIT']     = $renegociacao['CDFILIEMIT'];
            $row['CDEMPRESAEMIT']  = $renegociacao['CDEMPRESAEMIT'];
            $row['CDINSCESTAEMIT'] = $renegociacao['CDINSCESTAEMIT'];
            $row['CDINSCMUNIEMIT'] = $renegociacao['CDINSCMUNIEMIT'];
            $row['CDCLIENTE']      = $renegociacao['CDCLIENTE'];
            $row['NRINSJURCLIE']   = $renegociacao['NRINSJURCLIE'];
            $row['IDTPIJURCLIE']   = $renegociacao['IDTPIJURCLIE'];
            $row['CDSERIE']        = $renegociacao['CDSERIE'];
            $row['CDTIPOOPER']     = $renegociacao['CDTIPOOPER'];
            $row['NRLICENCAUSO']   = null;
            $row['CDVENDEDOR']     = null;
            $row['IDSTATUS']       = 'A';
            $row['CDTPCTRECEB']    = $renegociacao['CDTPCTRECEB'];
            $row['CDCLASSFINA']    = $renegociacao['CDCLASSFINA'];
            $row['DSCONTAREC']     = isset($renegociacao['DSRENEGOCIA'])?$renegociacao['DSRENEGOCIA']:null;
            $row['CDREDUCONT']     = $renegociacao['CDCLASSFINA'];


            foreach($titulosReneg as $tituloReneg){
                $row['NRNOTAFISC']   = $this->titulosAReceberService->geraNovoCodigo($renegociacao['NRORG'], 'NRNOTAFISCREC', 1, $renegociacao['NRTAMNUMNFUT']);
                $row['DTPRIMVENC']   = $tituloReneg['DTVENBOLREC'];
                $row['DTEMISSAO']    = $tituloReneg['DTVENBOLREC'];
                $row['DTENTRSAID']   = $tituloReneg['DTVENBOLREC'];
                $row['DTATUAVENREC'] = $tituloReneg['DTVENBOLREC'];
                $row['VRNOTAFISC']   = $tituloReneg['VRFINALBREC'];
                $row['NRSEQVENREC']  = $tituloReneg['NRPARCBOLREC'];

                $rows[] = $row;
            }

        } catch (\Exception $e) {
            throw $e;
        }
        return $rows;
    }

    /**
     * Exclui Titulos Descontados
     * @param Data $dataset
     * @param Response $response
     */
    public function excluiTitulosDescontados(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $titulosDescontados = $rows[0];

            // Salva Titulo Descontado
            $this->titulosAReceberService->excluiTitulosDescontados(
                $titulosDescontados
            );

            // Registra Dados
            $response->addDataSet(new DataSet('salvaTituloDescontado', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Efetivar Recebimento Titulos Descontados
     * @param Data $dataset
     * @param Response $response
     */
    public function efetivarRecebimentoDescontado(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $titulosDescontados = $rows[0];

            // Salva Titulo Descontado
            $this->titulosAReceberService->efetivarRecebimentoDescontado(
                $titulosDescontados
            );

            // Registra Dados
            $response->addDataSet(new DataSet('salvaTituloDescontado', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Efetivar Recebimento Titulos Descontados
     * @param Data $dataset
     * @param Response $response
     */
    public function cancelarRecebimento(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $titulosDescontados = $rows[0];

            // Salva Titulo Descontado
            $this->titulosAReceberService->cancelarRecebimento(
                $titulosDescontados
            );

            // Registra Dados
            $response->addDataSet(new DataSet('salvaTituloDescontado', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Cancela Renegociação Financeira
     * @param Data $dataset
     * @param Response $response
     */
    public function cancelaRenegociacaoFinanceira(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $titulosReneg = $rows[0];

            // Salva Titulo Descontado
            $this->titulosAReceberService->cancelaRenegociacaoFinanceira(
                $titulosReneg
            );

            // Registra Dados
            $response->addDataSet(new DataSet('cancelaRenegociacaoFinanceira', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Busca Todos Ajustes Boletos
     * @param Data $dataset
     * @param Response $response
     */
    public function buscaTodosAjustesBoletosRec(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $boletos  = $dataset->getDataSet()->getRows();

            $ajustesboletos = $this->titulosAReceberService->buscaTodosAjustesBoletosRec($boletos);
               
            $response->addDataSet(new DataSet('buscaTodosAjustesBoletosRec', $ajustesboletos));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function validaFilialLogadaPorEmpresaFiltrada(Row $row, Response $response){
        try {
            $row           = $row->getRow();
            $cdempresa     = $row['CDEMPRESA'];
            $cdfilial      = $this->environment->getCdFilial();
            $nrorg         = $this->environment->getNrOrgTrab();
            $filialEmpresa = $this->titulosAReceberService->validaFilialLogadaPorEmpresaFiltrada($cdempresa, $cdfilial, $nrorg);
            $filialEmpresa = ($filialEmpresa) ? [$filialEmpresa] : [];
            $response->addDataSet(new DataSet('filial', $filialEmpresa));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Gera Titulo a Receber
     * @param Row $row
     * @param Response $response
     */
    public function geraTituloAReceberDoc(Data $dataset, Response $response)
    {
        try {
            
            $dataset  = $dataset->getDataSet()->getRows();
            
            // Dados do Registro
            $row = $dataset[0];
            $anexos = $dataset[2];
            $permiteParcelamento = isset($row["PARCELAMENTO"]);
            
            $dataset  = $dataset[1];
            $row['DTATUAVENREC'] = $dataset[0]['DTVENBOLREC'];
            $row['DSCONTAREC']   = $dataset[0]['DSCONTAREC'];
            
            // Configura a row para enviar para o service
            $row = self::buildTitulosAReceberRow($row);
            
            if(count($dataset) > 1){
    
                // Variáveis de Inicialização
                $nrorg       = null;
                $cdfilial    = null;
                $nrlanbolrec = null;
                $parcelas = array();
                
                // Percorre Parcelas
                foreach($dataset as $data){
                    // Verifica Parcela Inicial
                    if(($data['NRPARCBOLREC'] == '00') || ($data['NRPARCBOLREC'] == '01')) {
                        $nrorg       = $row['NRORG'];
                        $cdfilial    = $row['CDFILIEMIT'];
                        $nrlanbolrec = $row['NRLANBOLREC'];
                    }
    
                    // Monta Array Parcelas
                    $parcelas[] = array(
                        'NRORG'            => $row['NRORG']             ,
                        'CDFILIAL'         => $row['CDFILIEMIT']        ,
                        'NRLANBOLREC'      => null                      ,
                        'NRORIGBOLREC'     => $row['NRLANBOLREC']       ,
                        'NRPARCBOLREC'     => $data['NRPARCBOLREC']     ,
                        'NRPARCBOLRECVISU' => $data['NRPARCBOLRECVISU'] ,
                        'DTVENBOLREC'      => $data['DTVENBOLREC']      ,
                        'DSCONTAREC'       => $data['DSCONTAREC']       ,
                        'VRFINALBREC'      => floatval($data['VRFINALBREC']),
                        'CDMOTIPARC'       => isset($data['CDMOTIPARC'])?$data['CDMOTIPARC']:null,
                        'IDSTATBREC'       => 'A'
                    );
                    
                }
                
                // Salva Titulo a Receber e Gera Parcelamento Título
                $this->titulosAReceberService->geraTituloAReceberDoc(
                    $row, $response, $permiteParcelamento, $nrorg, $cdfilial, $nrlanbolrec, $parcelas
                );
            }else{
                // Salva Titulo a Receber
                $this->titulosAReceberService->geraTituloAReceber(
                    $row, $response, $permiteParcelamento
                );  
            }
            
            if(!empty($anexos)){
                $this->titulosAReceberService->salvaAnexoInFileServer($anexos, $row['CDFILIEMIT'], $row['NRLANBOLREC']);
            }
            
            // Registra Dados
            $response->addDataSet(new DataSet('geraTituloAReceberDoc', ['nrlanbolrec' => $row['CDFILIEMIT'].$row['NRLANBOLREC']]));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Confirma Cálculo de Ajuste
     * @param Row $row
     * @param Response $response
     */
    public function confirmaCalculoAjuste(Data $dataset, Response $response)
    {
        try {
            $dataset  = $dataset->getDataSet()->getRows();

            $nrorg  = $this->environment->getNrOrgTrab();
            $titulo = $dataset[0];
            $ajuste = $dataset[1];
            
            if($ajuste['IDJUROS'] == 'S'){
                $this->titulosAReceberService->calcJurosSimples(
                    $nrorg, $titulo, $ajuste
                );
            }else{
                $this->titulosAReceberService->calcJurosComposto(
                    $nrorg, $titulo, $ajuste
                );
            }
            
            $response->addDataSet(new DataSet('confirmaCalculoAjuste', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * Salva Anexo do Titulo a Receber antes de salvar no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function salvaAnexoTituloRec(Row $row, Response $response)
    {
        try {
            
            $row  = $row->getRow();
            
            $this->titulosAReceberService->salvaAnexoTitulo($row);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Salva Anexo do Titulo a Receber no file Server
     * @param Row $row
     * @param Response $response
     */
    public function salvaAnexoGridRec(Row $row, Response $response)
    {
        try {
            
            $row  = $row->getRow();
            
            $this->titulosAReceberService->salvaAnexoGrid($row);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Anexo do Titulo a Receber antes de salvar no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function deletaAnexoTituloRec(Data $dataset, Response $response)
    {
        try {
            
            $rows = $dataset->getDataSet()->getRows();
            
            $this->titulosAReceberService->deletaAnexoTitulo($rows);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Anexo do Titulo a Receber e seu arquivo no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function deleteAnexoGridRec(Data $dataset, Response $response)
    {
        try {
            
            $rows = $dataset->getDataSet()->getRows();
            
            $this->titulosAReceberService->deleteAnexoGrid($rows);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Anexo do Titulo a Receber que restou como "lixo de memoria"
     * @param Row $row
     * @param Response $response
     */
    public function deleteAnexoIncompletosRec()
    {
        try {
            
            $this->titulosAReceberService->deleteAnexoIncompletos();
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Busca a url do FileServer
     * @param Row $row
     * @param Response $response
     */
    public function getFileServerUrlRec(Row $request, Response $response) {
        try {
            
            $fileName = $this->titulosAReceberService->getFileServerUrl();
            
            $response->addDataset(new DataSet('sucess', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    


    /**
     * Envia Carta de Cobrança por E-mail
     * @param Data $dataset
     * @param Response $response
     */
    public function enviaCartaCobrancaPorEmail(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $boletos = $rows[0];
            $envio   = $rows[1];

            $remetente = $envio['DSEMAILFILI'];
            $assunto   = $envio['DSASSUEMAIL'];
            $mensagem  = $envio['DSMENSEMAIL'];
            
            // Envia Título Por Email
            $this->titulosAReceberService->enviaCartaCobrancaPorEmail(
                $remetente, $assunto, $mensagem, $boletos, $dataset, $response
            );

            // Registra Dados
            $response->addDataSet(new DataSet('enviaCartaCobrancaPorEmail', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Envia Carta de Cobrança por E-mail
     * @param Data $dataset
     * @param Response $response
     */
    public function buscaContaCorrenteVinculada(Row $row, Response $response) {
        try {
            $row  = $row->getRow();
            $cdReduCont           = $row["cdReduCont"];
            $buscaFilialContaCorrente = $this->titulosAReceberService->buscaContaCorrenteVinculada($cdReduCont);
            $response->addDataSet(new DataSet('filial_conta_corrente', ['filial_conta_corrente' => $buscaFilialContaCorrente]));
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Preparação Títulos a Receber
     * @param Data $dataset
     * @param Response $response
     */
    public function preparacaoTitulosReceber(Data $dataset, Response $response) {
        try {
            $rows = $dataset->getDataSet()->getRows();

            $dados   = $rows[0];
            $titulos = $rows[1];
            
            $this->titulosAReceberService->preparacaoTitulosReceber(
                $dados, $titulos
            );
            
            $response->addDataSet(new DataSet('preparacaoTitulosReceber', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}